use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006000
eval "use Test::Spelling 0.12; use Pod::Wordlist::hanekomu; 1" or die $@;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
AMD
IDentifier
Mersenne
OSX
UUID
UUIDs
benchmarking
cryptographically
libuuid
multicast
timestamp
David
Golden
dagolden
Matt
Koscica
matt
lib
Data
MT
